/// @file       HwAdapt.c
/// @copyright  Copyright (c) 2012 Toradex AG \n
///             All information contained herein is, and remains the property of Toradex Ag and its suppliers, if any. The intellectual\n
///             and technical concepts contained herein are proprietary to Toradex AG and its suppliers and may be covered by Swiss and\n
///             foreign patents, patents in process, and are protected by trade secret or copyright law. Dissemination of this information\n
///             or reproduction of this material is strictly forbidden unless prior written permission is obtained from Toradex AG.\n
/// @author     $Author$
/// @version    $Rev$
/// @date       $Date$
/// @brief      The hardware adaption of the multi touch driver solution is made to run as an application.\n
///             The advantage is that a customer can modify, run and test the "driver" like a normal application. \n
///             The "hardware adaption" handles the whole interface to the touch controller (interrupt, I2C communication etc.)
///             using the standard Toradex library which are free available. Therefore it is much easier for a customer to adapt this
///             "hardware adaption" to other touch controller. \n
///             The counterpart to this "hardware adaption" is the "unified driver" which must be installed like a normal driver. \n
///             The "unified driver" has no connection to the hardware and run on all colibri modules and WinCE versions.
/// @target     Colibri T20, T30, WinEC7
/// @caveats    No screen output. All messages are send to the debug port.\n
///             This file contains the main loop.\n
///
#include <windows.h>
#include <Winbase.h>
#include <Winuser.h>
#include <winioctl.h>
#include <stdio.h>
#include "StdAfx.h"
#include "TchContr_FT5x06.h"


//==== Debug Zones ================================================================================
// simple handling: debug zones are forced with TREU or FALSE and need a recompile to be activated
// for sophisticated handling: use macro DEBUGZONE(), DBGPARAM dpCurSettings etc.
// (see MSDN e.g. Win CE 5.0, Debug Zones)
#undef ZONE_ERROR
#undef ZONE_WARN
#undef ZONE_FUNCTION
#undef ZONE_INFO
#undef ZONE_TIPSTATE

#define ZONE_ERROR      TRUE
#define ZONE_WARN       TRUE
#define ZONE_FUNCTION   FALSE
#define ZONE_INFO       FALSE
#define ZONE_TIPSTATE   FALSE

//=================================================================================================
/// _tmain
int WINAPI WinMain( HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPTSTR    lpCmdLine,
                    int       nCmdShow)
{
    HANDLE          hDS;                                                // handle for "unified driver"
    CETCHINP_IOCTL  CeTchInp;                                           // touch events packed for IOCTL
    DWORD           BytesHandled;                                       // dummy for DeviceIoControl()
    BytesHandled    = 0;                                                // to avoid warning

    RETAILMSG(1,(TEXT("MultiTchHwAdapt loading...")));

    // Read the and setup the configuration
    vI2C_ReadConfiguration();                                           // read static global configuration parameters (default or set in the regitry)

    // Init the IC interface on the Colibri (makes no sense to retry)
    if (FALSE==bI2C_Init())                                             // init I2C communication and setup interrupt handling
    {
        RETAILMSG(1,(TEXT("aborted\r\n")));                             // bI2C_Init() printed the detail of the failure already
        vI2C_DeInit();                                                  // release anything active already
        return 0;
    }

    // Get a handle to the driver. Which causes the driver's XXX_Open function to be called
    hDS = CreateFile( TEXT("TCH1:"), GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0);
    if (INVALID_HANDLE_VALUE==hDS)
    {
        RETAILMSG(1,(TEXT("no Unified Touch Driver...")));
        RETAILMSG(1,(TEXT("aborted\r\n")));                             // bI2C_Init() printed the detail of the failure already
        vI2C_DeInit();                                                  // release anything active already
        return 0;
    }


    // main loop
    // The used touch controller is setup to set the interrupt line active if one or more fingers touch the screen.\n
    // If the touch area is untouched then the interrupt is set to passive. \n
    // This thread mainly sleeps until a timeout occurs. If this happen the interrupt signal is tested. \n
    // If the interrupt line is active (touches) then the status, current position etc. are read out\n
    // of the touch controller and are processed. Afterwards the next timeout is set to the sample rate\n
    // and the thread is going to sleep again. \n
    // If the interrupt line is passive (no touches) then the communication to the touch controller is checked.\n
    // Afterwards the next timeout is set to the communication test rate. \n
    // Beside that, a transition of the interrupt line generates a interrupt event which wakeup the thread immediately.\n
    // This shorten up the reaction time for the first touch.

    // main loop
    while (1)
    {
        //------ Startup Procedure --------------------------------------------------------------------

        DEBUGMSG(ZONE_INFO,(TEXT("MultiTchHwAdapt: Start communication with the touch controller FT5x06\r\n")));

        if (FALSE==bInitFT5x06())                                       // reset and start touch controller
        {
            goto restart;
        }

        RETAILMSG(TRUE,(TEXT("done\r\n")));                             // app driver runs

        // wait for messages
        while(1)
        {
            if (FALSE==bWaitTouchDataINT(dwCOMTST_INT))
            {
                if (FALSE==bCommunicationTestFT5x06())                  // timeout, test the communication to the touch controller
                {
                    goto restart;
                }
            }
            else                                                        // real interrupt
            {
                // get the touch events
                vTouchPanelGetPoint(&CeTchInp.TchInp[0],&CeTchInp.iCnt); // get the touch events from the controller

                if ((1==CeTchInp.iCnt)||(2==CeTchInp.iCnt))
                {
                    // send the touch events to the unified touch driver
                    DeviceIoControl( hDS, IOCTL_SET_TOUCH_EVENT, &CeTchInp, sizeof(CeTchInp), NULL, 0, &BytesHandled, NULL );
                }
            }
        }

        // restart sequence
restart:
        RETAILMSG(1,(TEXT("MultiTchHwAdapt: restarts communication with touch controller...\r\n")));

        vDeInitFT5x06();                                                // stop (reset/powerdown) the Touch Controller

        Sleep(400);                                                     // wait before an new try starts
    }
    return 0;
}