/// @file       TchContr_FT5x06.c
/// @copyright  Copyright (c) 2012 Toradex AG \n
///             All information contained herein is, and remains the property of Toradex Ag and its suppliers, if any. The intellectual\n
///             and technical concepts contained herein are proprietary to Toradex AG and its suppliers and may be covered by Swiss and\n
///             foreign patents, patents in process, and are protected by trade secret or copyright law. Dissemination of this information\n
///             or reproduction of this material is strictly forbidden unless prior written permission is obtained from Toradex AG.\n
/// @author     $Author$
/// @version    $Rev$
/// @date       $Date$
/// @brief      Function library for the touch controller FT5x06
/// @target     Colibri PXA270, PX3xx, T20 WinCE6, WinEC7

#include <windows.h>
#include <Windef.h>
#include <limits.h>
#include "IntLib.h"
#include "i2cLib.h"
#include "GpioLib.h"
#include "TchContr_FT5x06.h"

// ==== Debug Zones ================================================================================
// simple handling: debug zones are forced with TREU or FALSE and need a recompile to be activated
// for sophisticated handling: use macro DEBUGZONE(), DBGPARAM dpCurSettings etc.
// (see MSDN e.g. Win CE 5.0, Debug Zones)
#undef ZONE_ERROR
#undef ZONE_WARN
#undef ZONE_FUNCTION
#undef ZONE_INFO
#undef ZONE_TIPSTATE

#define ZONE_ERROR      TRUE
#define ZONE_WARN       TRUE
#define ZONE_FUNCTION   FALSE
#define ZONE_INFO       FALSE
#define ZONE_TIPSTATE   FALSE

//== General ======================================================================================
//----- private preprocessor definitions -----------------------------------------------------------

//----- private members ----------------------------------------------------------------------------

//== I2C Communication ============================================================================

//----- private preprocessor definitions -----------------------------------------------------------
#define dwI2C_I2C_ADR                   0x38                            // I2C address of the FT5x06 chip

#define dwINT_SODIMM_PIN                133                             // default SODIMM pin for the "INT Line"
#define dwINT_SIG_INV                   0                               // 0 => INT signal, Falling Edge is interrupt
                                                                        // >0 => INT signal Rising Edge is interrupt
#define dwRESET_SODIMM_PIN              127                             // default SODIMM pin for the "RESET Line"
#define dwRESET_POSTDLY                 300                             // delay after the "RESET Line" is released
#define dwRESET_LIN_INV                 1                               // 0 => "RESET Line" reset active is High and reset off is Low
                                                                        // >0 => "RESET line" reset active is Low and reset off is High

#define dwI2C_100KB                     1                               // 100kb I2C speed
#define dwI2C_400KB                     2                               // 400kb I2C speed
#define HWADAPTPRIO                     40                              // task priority hardware adapter




//== "CTP Register Copy" ==========================================================================
//----- private preprocessor definitions -----------------------------------------------------------

// Work Mode Registers  (Register definition see FTS_AN_CTPM_Standard_eng_ver1.1.pdf)
#define WMREGSIZE                       0x100                           // size of register block

// DEVIDE_MODE
#define DEVICE_MODE_ADR                 0x00                            // register address
#define DEVICE_MODE_SIZE                1                               // count of bytes

#define bDEVICE_MODE_Work_TST()         ((ucWMRegister[DEVICE_MODE_ADR]&0x70)==0x00) // access macros
#define bDEVICE_MODE_Factory_TST()      ((ucWMRegister[DEVICE_MODE_ADR]&0x70)==0x40)

//GEST_ID
#define GEST_ID_ADR                     0x01                            // register address
#define GEST_ID_SIZE                    1                               // count of bytes

#define ucGEST_ID_GET()                 ucWMRegister[GEST_ID_ADR]       // access macros
#define GEST_ID_NoGesture               0x00
#define GEST_ID_MoveUp                  0x10
#define GEST_ID_MoveLeft                0x14
#define GEST_ID_MoveDown                0x18
#define GEST_ID_MoveRight               0x1C
#define GEST_ID_ZoomIn                  0x48
#define GEST_ID_ZoomOut                 0x49

// TD_STATUS
#define TD_STATUS_ADR                   0x02                            // register address
#define TD_STATUS_SIZE                  1                               // count of bytes

#define ucTD_STATUS_GET()               (ucWMRegister[TD_STATUS_ADR]&0x0F) // access macros

// TOUCHx (tnr = touch number 1 to 10)
#define TOUCH1_ADR                      0x03                            // register address
#define TOUCHx_SIZE                     0x6                               // count of bytes

#define ucTOUCHx_EVENT_FLG_GET(tnr)     (ucWMRegister[TOUCH1_ADR+((tnr-1)*TOUCHx_SIZE)]>>6) // access macros for Event Flags
#define TOUCHx_PutDown                  0x00
#define TOUCHx_PutUP                    0x01
#define TOUCHx_Contact                  0x02

#define ucTOUCHx_ID_GET(tnr)            (ucWMRegister[TOUCH1_ADR+((tnr-1)*TOUCHx_SIZE)+2]>>4) // access macros for Touch ID


#define uiTOUCHx_X_GET(tnr)            ((ucWMRegister[TOUCH1_ADR+((tnr-1)*TOUCHx_SIZE)]&0x0F)*256)+ucWMRegister[TOUCH1_ADR+((tnr-1)*TOUCHx_SIZE)+1]
#define uiTOUCHx_Y_GET(tnr)            ((ucWMRegister[TOUCH1_ADR+((tnr-1)*TOUCHx_SIZE)+2]&0x0F)*256)+ucWMRegister[TOUCH1_ADR+((tnr-1)*TOUCHx_SIZE)+3]


#define TOUCH1_XH                       0x03
#define TOUCH1_XL                       0x04
#define TOUCH1_YH                       0x05
#define TOUCH1_YL                       0x06

#define TOUCH2_XH                       0x09
#define TOUCH2_XL                       0x0A
#define TOUCH2_YH                       0x0B
#define TOUCH2_YL                       0x0C

#define TOUCH3_XH                       0x0F
#define TOUCH3_XL                       0x10
#define TOUCH3_YH                       0x11
#define TOUCH3_YL                       0x12

#define TOUCH4_XH                       0x15
#define TOUCH4_XL                       0x16
#define TOUCH4_YH                       0x17
#define TOUCH4_YL                       0x18

#define TOUCH5_XH                       0x1B
#define TOUCH5_XL                       0x1C
#define TOUCH5_YH                       0x1D
#define TOUCH5_YL                       0x1E

#define TOUCH6_XH                       0x21
#define TOUCH6_XL                       0x22
#define TOUCH6_YH                       0x23
#define TOUCH6_YL                       0x24

#define TOUCH7_XH                       0x27
#define TOUCH7_XL                       0x28
#define TOUCH7_YH                       0x29
#define TOUCH7_YL                       0x2A

#define TOUCH8_XH                       0x2D
#define TOUCH8_XL                       0x2E
#define TOUCH8_YH                       0x2F
#define TOUCH8_YL                       0x30

#define TOUCH9_XH                       0x33
#define TOUCH9_XL                       0x34
#define TOUCH9_YH                       0x35
#define TOUCH9_YL                       0x36

#define TOUCH10_XH                      0x39
#define TOUCH10_XL                      0x3A
#define TOUCH10_YH                      0x3B
#define TOUCH10_YL                      0x3C

#define ID_G_THGROUP                    0x80
#define ID_G_THPEAK                     0x81
#define ID_G_THCAL                      0x82
#define ID_G_THWATER                    0x83
#define ID_G_THTEMP                     0x84

#define ID_G_CTRL                       0x86
#define ID_G_TIME_ENTER                 0x87
#define ID_G_PERIODACTIVE               0x88
#define ID_G_PERIOD                     0x89

#define ID_G_AUTO_CLB                   0xA0
#define ID_G_LIB_                       0xA1
#define ID_G_LIB                        0xA2
#define ID_G_CIPHER                     0xA3


// ID_G_MODE
#define ID_G_MODE_ADR                   0xA4                            // register address
#define ID_G_MODE_SIZE                  1                               // count of bytes
#define ID_G_MODE_Polling               0                               // polling mode
#define ID_G_MODE_Trigger               1                               // trigger mode


#define ucID_G_MODE_GET()               ucWMRegister[ID_G_MODE_ADR]     // access macros
#define vID_G_MODE_SET(mode)            ucWMRegister[ID_G_MODE_ADR]=mode // access macros

#define ID_G_PMODE                      0xA5
#define ID_G_FIRMID                     0xA6
#define ID_G_STATE                      0xA7
#define ID_G_FT5201ID                   0xA8
#define ID_G_ERR                        0xA9
#define ID_G_CLB                        0xAA

#define ID_G_B_AREA_TH                  0xAE

#define LOG_MSG_CNT                     0xFE
#define LOG_CUR_CHA                     0xFF

// Register sizes (see FTS_AN_CTPM_Standard_eng_ver1.1.pdf)
#define TOUCHREGBLK_SIZE                 4                               // register block size for touches

#define     TOUCH_ID_1      0
#define     TOUCH_ID_2      1

//----- priver members ----------------------------------------------------------------------------
UCHAR                                   ucWMRegister[WMREGSIZE];        // copy of Work Mode Registers

//==== privat type definitions ====================================================================
typedef struct
{
    UINT8   u8NuFinger;                                                 // Number of Fingers

    // first touch point information
    UINT16  u16Px1;                                                     // X Position
    UINT16  u16Py1;                                                     // Y Position
    BOOL    bTipSwitch1;                                                // Tip Switch
    UINT8   u8TouchID1;                                                 // Touch ID

    // second touch point information
    UINT16  u16Px2;                                                     // X Position
    UINT16  u16Py2;                                                     // Y Position
    BOOL    bTipSwitch2;                                                // Tip Switch
    UINT8   u8TouchID2;                                                 // Touch ID
}I2C_DEVICE;

static DWORD                            dwI2C_Speed;
static DWORD                            dwI2C_Adr;
static DWORD                            dwINT_SODIMMPin;
static DWORD                            dwINT_Sig_Inv;
static DWORD                            dwRESET_SODIMMPin;
static DWORD                            dwRESET_PostDly;
static DWORD                            dwRESET_Lin_Inv;
static DWORD                            dwINT_GPIONumber;
static DWORD                            dwRESET_GPIONumber;
static HANDLE                           hI2C_Event;                     // for "interrupts"
static DWORD                            dwI2C_SysIntr;
static DWORD                            dwHwAdaptPrio;                  // task priority


//----- privat methods definition -----------------------------------------------------------------
// @internal
/// Wait for Interrupt to goes to In active state with timeout
/// @param[in]    DWORD dwTimeout
/// @param[out]   none
/// @retval       none
/// @retval       none
/// @caveats      none
static void WaitForInterruptInActiveState(DWORD dwTimeout)
{
    while(dwTimeout--)
    {
        if (0==dwINT_Sig_Inv)                       // interrupt input is not inverted
        {
            if(1==GetGPIOLevel(dwINT_GPIONumber))   // interrupt input already low means there are pending touches
                Sleep(1);
            else
                break;
        }
        else
        {
            if(0==GetGPIOLevel(dwINT_GPIONumber))   // interrupt input already low means there are pending touches
                Sleep(1);
            else
                break;
        }

    }

}

// @internal
/// Read out parts of "Work Mode Registers" out of the touch controller FT5x06 and place in the internal copy (array).\n
/// The index of this array is equal to the addresses of the "Work Mode Registers" in the touch controller.
/// @param[in]    sStartRegister  The device's i2c address (0..127)
/// @param[in]    iNumber  Number of bytes to read
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE error, a message to the console shows the reason
/// @caveats      none
static BOOL bReadInRegister(SHORT sStartRegister,UINT iNumber)
{
    BOOL bSuccess;
    LONG lErr;
    DWORD dwAlignBuffer[WMREGSIZE/4+1];     // the Tegra I2C library needs an alignment of 4 for the buffer
                                            // therefore all I2C read or write use the alignment buffer and copy it to the not alignment place
    bSuccess = FALSE;

    if (FALSE==GetI2CLock(1000))
    {
        lErr = GetLastError();
        RETAILMSG (TRUE,(TEXT("\r\nillegal response in GetI2Lock() %d"),lErr));
        Sleep(50);
    }
    else
    {
        if (FALSE == I2CBurstRead((UCHAR)dwI2C_Adr,(UCHAR*)dwAlignBuffer,sStartRegister,iNumber))
        {
            lErr = GetLastError();
            RETAILMSG (TRUE,(TEXT("\r\nillegal response in I2CBurstRead() %d"),lErr));
        }
        else
        {
            bSuccess = TRUE;
            memcpy(&ucWMRegister[sStartRegister],dwAlignBuffer,iNumber);    // copy the alignment buffer to original place
        }
        ReleaseI2CLock();
    }

    return bSuccess;
}

// @internal
/// Write out parts of internal register copy (array) to the "Work Mode Registers" of the touch controller FT5x06
/// The index of this array is equal to the adrresses of the "Work Mode Registers" in the touch controller
/// @param[in]    sStartRegister    The device's i2c address (0..127)
/// @param[in]    iNumber   Number of bytes to read
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE error, a message to the console shows the reason
/// @caveats      none
static BOOL bWriteOutRegister(SHORT sStartRegister,UINT iNumber)
{
    BOOL bSuccess;
    LONG lErr;

    DWORD dwAlignBuffer[WMREGSIZE/4+1];     // the Tegra I2C library needs an alignment of 4 for the buffer
                                            // therefore all I2C read or write use the alignment buffer and copy it to the not alignment place

    memcpy(dwAlignBuffer,&ucWMRegister[sStartRegister],iNumber);    // copy the alignment buffer to original place

    bSuccess = FALSE;

    if (FALSE==GetI2CLock(1000))
    {
        lErr = GetLastError();
        RETAILMSG (TRUE,(TEXT("\r\nillegal response in GetI2Lock() %d"),lErr));
        Sleep(50);
    }
    else
    {
        if (FALSE == I2CBurstWrite((UCHAR)dwI2C_Adr,(UCHAR*)dwAlignBuffer,sStartRegister,iNumber))
        {
            lErr = GetLastError();
            RETAILMSG (TRUE,(TEXT("\r\nillegal response in I2CBurstWrite() %d"),lErr));
        }
        else
        {
            bSuccess = TRUE;
        }

        ReleaseI2CLock();
    }

    return bSuccess;
}

// @internal
/// Read out all the registers containing touch data of the touch controller
/// @param[in]    psI2CDat = pointer to the touch events data
/// @param[out]   psI2CDat = pointer to the touch events data (in case of error the data are set to no touches)
/// @retval       none
/// @caveats      none
static void vGetControllerData(I2C_DEVICE *psI2CDat)
{
    DEBUGMSG(ZONE_FUNCTION, ( TEXT("MultiTchHwAdapt:vGetControllerData()+\r\n" )) );

    if (FALSE==bUpdateTouchCategoryFT5x06())                // update the local copy of the registers
    {
        ERRORMSG(ZONE_ERROR, (TEXT("MultiTchHwAdapt:vGetControllerData() I2C read failed\r\n")));
        psI2CDat->u8NuFinger    = 0;                                    // clean all data
        psI2CDat->u16Px1        = 0;
        psI2CDat->u16Py1        = 0;
        psI2CDat->bTipSwitch1   = FALSE;
        psI2CDat->u8TouchID1    = 0;
        psI2CDat->u16Px2        = 0;
        psI2CDat->u16Py2        = 0;
        psI2CDat->bTipSwitch2   = FALSE;
        psI2CDat->u8TouchID2    = 0;
        goto out;
     }

    psI2CDat->u8NuFinger    = (INT)ucTD_STATUS_GET();                   // Number of Fingers

    psI2CDat->u16Px1        = uiTOUCHx_X_GET(1);
    psI2CDat->u16Py1        = uiTOUCHx_Y_GET(1);

    psI2CDat->bTipSwitch1   = ucTOUCHx_EVENT_FLG_GET(1);                // Tip Switch (TRUE = finger on touch)
    psI2CDat->bTipSwitch1   = psI2CDat->bTipSwitch1 ^ 0x01;             //alter bit 0 state, as we are UP as O and Down as 1.
    psI2CDat->u8TouchID1    = ucTOUCHx_ID_GET(1);                       // Touch ID

    psI2CDat->u16Px2        = uiTOUCHx_X_GET(2);
    psI2CDat->u16Py2        = uiTOUCHx_Y_GET(2);

    psI2CDat->bTipSwitch2   = ucTOUCHx_EVENT_FLG_GET(2);                // Tip Switch
    psI2CDat->bTipSwitch2   = psI2CDat->bTipSwitch2 ^ 0x01;             //alter bit 0 state, as we are calculating UP as O and Down as 1.
    psI2CDat->u8TouchID2    = ucTOUCHx_ID_GET(2);                       //Touch ID

    //Touch up event is missing, when touch up u8NuFinger number is wrong. So Counting number of fingers by detecting valid touch ID.
    if(psI2CDat->u8TouchID1 != 15)
        psI2CDat->u8NuFinger = 1;

    if(psI2CDat->u8TouchID2 != 15)
        psI2CDat->u8NuFinger = 2;
out:
    DEBUGMSG(ZONE_FUNCTION,(TEXT("MultiTchHwAdapt:vGetControllerData() -\r\n" )));
    return;
}



//----- public methods definition -----------------------------------------------------------------
/// Set the configuration data according to the registry entry or to the default value
/// These data are "INT port number", "RESET port number", "I2C slave address", "I2C speed",
/// The values are set to default or set by the entry in the registry (if a valid entry exists)
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @caveats      none
void vI2C_ReadConfiguration(void)
{
    HKEY    hKey;
    DWORD   dwSize;
    LONG    lError;

    // set default registry values
    dwI2C_Speed         = dwI2C_100KB;                                  // I2C speed
    dwI2C_Adr           = dwI2C_I2C_ADR;                                // I2C address of the FT5x06 chip

    dwINT_SODIMMPin     = dwINT_SODIMM_PIN;                             // SODIMM pin for Int
    dwINT_Sig_Inv       = dwINT_SIG_INV;                                // Int signal invertetd
    dwRESET_SODIMMPin   = dwRESET_SODIMM_PIN;                           // SODIMM pin for Reset
    dwRESET_PostDly     = dwRESET_POSTDLY;                              // delay after the "RESET Line" is released
    dwRESET_Lin_Inv     = dwRESET_LIN_INV;                              // Reset line inverted
    dwHwAdaptPrio       = HWADAPTPRIO;                                  // Task priority
    dwINT_GPIONumber    = 0;                                            // GPIO pin for Int
    dwRESET_GPIONumber  = 0;                                            // GPIO pin for Reset

    // lock if there are settings in the registry
    lError = RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"Software\\Toradex\\MultiTchHwAdapt", 0, KEY_ALL_ACCESS, &hKey);
    if (lError == ERROR_SUCCESS)                                        // only if the registry key exists
    {
        dwSize=sizeof(DWORD);
        RegQueryValueEx(hKey, L"Int_SODIMMPin", NULL, NULL, (LPBYTE)&dwINT_SODIMMPin, &dwSize); // get the GPIONumber for interrupt
        RegQueryValueEx(hKey, L"Int_Signal_Inv", NULL, NULL, (LPBYTE)&dwINT_Sig_Inv, &dwSize); // get the GPIONumber for interrupt active level
        RegQueryValueEx(hKey, L"Reset_SODIMMPin", NULL, NULL, (LPBYTE)&dwRESET_SODIMMPin, &dwSize); // get the GPIONumber for reset
        RegQueryValueEx(hKey, L"Reset_Post_Delay", NULL, NULL, (LPBYTE)&dwRESET_PostDly, &dwSize); // get the pso delay for reset
        RegQueryValueEx(hKey, L"Reset_Line_Inv", NULL, NULL, (LPBYTE)&dwRESET_Lin_Inv, &dwSize); // get the inversion for reset
        RegQueryValueEx(hKey, L"I2CSpeed", NULL, NULL, (LPBYTE)&dwI2C_Speed,&dwSize); // get the I2C Speed
        RegQueryValueEx(hKey, L"I2CAddress", NULL, NULL, (LPBYTE)&dwI2C_Adr,&dwSize); // get the I2C Speed
        RegQueryValueEx(hKey, L"HwAdaptPriority", NULL, NULL, (LPBYTE)&dwHwAdaptPrio,&dwSize); // get the priority
        RegCloseKey (hKey);
    }
}

/// Init the I2C communication and the GIPO for the "INT" and "RST" pin of the touch controller
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE error, a message to the console shows the reason
/// @caveats      vI2C_ReadConfiguration() has to be executed
BOOL bI2C_Init(void)
{
    BOOL        bSuccess;
    DWORD       dwVerMaj;
    DWORD       dwVerMin;
    DWORD       dwVerBuild;
    DWORD       dwIRq;
    DWORD       dwEdge;

    PIN_INSTANCE sGPIO;

    //------ I2C init ----------------------------------------------------------------------------
    I2cIntGetLibVersion(&dwVerMaj, &dwVerMin, &dwVerBuild);

#if 0
    RETAILMSG (TRUE,(TEXT("\r\nI2C Library Version=%d.%d Build=%d\r\n"),dwVerMaj, dwVerMin, dwVerBuild));
#endif

    bSuccess = FALSE;

    if (TRUE==I2CInit())
    {
        switch (dwI2C_Speed)
        {
            case dwI2C_100KB:
                bSuccess = GetI2CLock(500);
                if (TRUE==bSuccess)
                {
                    I2CSetSpeed(I2C_100KBPS);                           // set speed
                    ReleaseI2CLock();
                }
                break;

            case dwI2C_400KB:
                bSuccess = GetI2CLock(500);
                if (TRUE==bSuccess)
                {
                    I2CSetSpeed(I2C_400KBPS);                               // set speed
                    ReleaseI2CLock();
                }
                break;

            default:
                break;
        }
    }

    //------ GPIO init ----------------------------------------------------------------------------
    if (TRUE==bSuccess)
    {
        bSuccess = FALSE;

        if (TRUE == GetGPIOFromPin(dwINT_SODIMMPin, FALSE, &sGPIO))
        {
            if (0xFF != sGPIO.inst2)                                    // function on a second GPIO exist
            {
                SetGPIOAltFn((DWORD)sGPIO.inst2, -1, DIR_IN);           // set it to input
            }
            dwINT_GPIONumber = sGPIO.inst1;                             // use inst1 as GPIONumber

            if (TRUE == GetGPIOFromPin(dwRESET_SODIMMPin, FALSE, &sGPIO))
            {
                if (0xFF != sGPIO.inst2)                                // function on a second GPIO exist
                {
                    SetGPIOAltFn((DWORD)sGPIO.inst2, -1, DIR_IN);       // set it to input
                }
                dwRESET_GPIONumber = sGPIO.inst1;                       // use inst1 as GPIONumber
                bSuccess = TRUE;
            }
            else
            {
                RETAILMSG (TRUE,(TEXT("\r\nlook up Reset_GPIONumber fails")));
            }
        }
        else
        {
            RETAILMSG (TRUE,(TEXT("\r\nlook up Int_GPIONumber fails")));
        }
    }

    if (TRUE==bSuccess)
    {
        bSuccess = FALSE;

        SetGPIOAltFn(dwRESET_GPIONumber, -1, DIR_OUT);
        if (0==dwRESET_Lin_Inv)                                         // Reset not inverted
        {
            SetGPIOLevel (dwRESET_GPIONumber,1);                        // Reset active / no power (vInitFT5x06())
        }
        else
        {
            SetGPIOLevel (dwRESET_GPIONumber,0);                        // Reset active / no power (vInitFT5x06())
        }

        SetGPIOAltFn(dwINT_GPIONumber, -1, DIR_IN);                     // Set INT as Input

        dwIRq=GetGPIOIrq(dwINT_GPIONumber);
        if(dwIRq)
        {
            // Set edge to trigger (Rising, Falling or both)
            if (0==dwINT_Sig_Inv)                                   // interrupt input is not inverted
            {
                dwEdge= GPIO_EDGE_FALLING;                          //falling edge
            }
            else
            {
                dwEdge= GPIO_EDGE_RISING;
            }
            if(SetGPIOIrqEdge(dwINT_GPIONumber,dwEdge))
            {
                // Create an event to wait on
                hI2C_Event = CreateEvent(NULL, FALSE, FALSE, NULL);
                if (hI2C_Event!= NULL)
                {
                    // Get the SYSINTR that corresponds to dwIrq (non GPIO Irqs correspond to the bit numbers of the PXA27x Interrupt resisters)
                    dwI2C_SysIntr=RequestSysInterrupt(dwIRq);

                    if (dwI2C_SysIntr)
                    {
                        if (InterruptInitializeCompat(dwI2C_SysIntr,hI2C_Event,NULL,0))
                        {
                            CeSetThreadPriority(GetCurrentThread(), dwHwAdaptPrio);
                            bSuccess = TRUE;
                        }
                    }
                }
            }
        }
    }
    if (FALSE==bSuccess)
    {
        RETAILMSG (TRUE,(TEXT("\r\nbI2C_Init() had failed")));
    }
    return bSuccess;
}

/// Stop the I2C communication (release interrupt, mutex, stop the I2C driver etc)
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @retval       none
/// @caveats      none
void vI2C_DeInit(void)
{
    // GPIO disable
    InterruptDoneCompat(dwI2C_SysIntr);
    InterruptDisableCompat(dwI2C_SysIntr);
    ReleaseSysIntr(dwI2C_SysIntr);

    // I2C driver
    ReleaseI2CLock();
    I2CDeInit();
}

/// Test the level of the INT line
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  INT line active
/// @retval       FALSE INT line passive
/// @caveats      none
BOOL bTstINTFT5x06(void)
{
    if (0==dwINT_Sig_Inv)                                           // interrupt input is not inverted
    {
        return (0==GetGPIOLevel(dwINT_GPIONumber));                 // interrupt input already low means there are pending touches
    }
    else
    {
        return (1==GetGPIOLevel(dwINT_GPIONumber));                 // interrupt input already high means there are pending touches
    }
}

/// Release the current interrupt and wait for the next interrupt with timeout
/// @param[in]    DWORD dwTimeout
/// @param[out]   none
/// @retval       TRUE  Interrupt
/// @retval       FALSE Timeout
/// @caveats      none
BOOL bWaitTouchDataINT(DWORD dwTimeout)
{
    InterruptDoneCompat(dwI2C_SysIntr); //clear the interrupt flag
    if (WAIT_TIMEOUT == WaitForSingleObject(hI2C_Event,dwTimeout))  // wait for input interrupt or timeout
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}

/// Start the touch controller (release the RST/PWR line) start the communication and and setup the\n
/// necessary registers in the touch controller
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE error, a message to the console shows the reason
/// @caveats      none
BOOL bInitFT5x06()
{
    UCHAR ucByte1;


    if (0==dwRESET_Lin_Inv)                                             // Reset not inverted
    {
        SetGPIOLevel (dwRESET_GPIONumber,0);                            // Reset passive / Power on)
    }
    else
    {
        SetGPIOLevel (dwRESET_GPIONumber,1);                            // Reset passive / Power on)
    }

    Sleep(dwRESET_PostDly);                                             // wait until the reset sequence of the touch controller is over

    // get the interrupt status
    if (FALSE==bReadInRegister(ID_G_MODE_ADR,ID_G_MODE_SIZE))
    {
        return FALSE;
    }

    // get the interrupt status/mode
    ucByte1 = ucID_G_MODE_GET();

#if 0
    // get more information like Firmware etc
#endif

    vID_G_MODE_SET(ID_G_MODE_Trigger);                                  // set Interrupt mode
    if (FALSE==bWriteOutRegister(ID_G_MODE_ADR,ID_G_MODE_SIZE))
    {
        return FALSE;
    }

    if (TRUE==bReadInRegister(ID_G_THGROUP,5))
    {
        DEBUGMSG(ZONE_INFO,(TEXT("ID_G_THGROUP(0x%X) ID_G_THPEAK(0x%X) ID_G_THCAL(0x%X) ID_G_THWATER(0x%X) ID_G_THTEMP(0x%X)\r\n" ),ucWMRegister[ID_G_THGROUP],ucWMRegister[ID_G_THPEAK],ucWMRegister[ID_G_THCAL],ucWMRegister[ID_G_THWATER],ucWMRegister[ID_G_THTEMP]));
    }
    //User may need to find optimum value to get precise and valid touch points using below touch controller registers
    /* ucWMRegister[ID_G_THGROUP] = 280/4;
    ucWMRegister[ID_G_THPEAK] = 60;
    ucWMRegister[ID_G_THCAL] = 128;
    ucWMRegister[ID_G_THWATER] = 60;
    ucWMRegister[ID_G_THTEMP] = 10;

    if(TRUE == bWriteOutRegister(ID_G_THCAL,1))
    {
        if (TRUE==bReadInRegister(ID_G_THGROUP,5))
        {
            RETAILMSG(1,(TEXT("ID_G_THGROUP(0x%X) ID_G_THPEAK(0x%X) ID_G_THCAL(0x%X) ID_G_THWATER(0x%X) ID_G_THTEMP(0x%X)\r\n" ),ucWMRegister[ID_G_THGROUP],ucWMRegister[ID_G_THPEAK],ucWMRegister[ID_G_THCAL],ucWMRegister[ID_G_THWATER],ucWMRegister[ID_G_THTEMP]));
        }
    }  */
    InterruptDoneCompat(dwI2C_SysIntr);
    return TRUE;
}

/// Reset on or power off the touch controller\n
/// Because the touch controller is mounted very closed to the display an ESD pulse could disturb the controller in a way that his is not working any more (not broken for ever)
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @caveats      none
void vDeInitFT5x06(void)                                                // stop (reset/powerdown) the Touch Controller
{
    if (0==dwRESET_Lin_Inv)                                             // Reset not inverted
    {
        SetGPIOLevel (dwRESET_GPIONumber,1);                            // Reset active or no power
    }
    else
    {
        SetGPIOLevel (dwRESET_GPIONumber,0);                            // Reset active or no power
    }

    ReleaseI2CLock();
}

/// Read out the count of touches (TD_STATUS) and the data for the 2 touch from the  touch controller\n
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE communication error with the touch controller, a message to the console shows the reason
/// @caveats      none
BOOL bUpdateTouchCategoryFT5x06(void)
{
    // get the status (number of touch points) and TOUCH1 and TOUCH2
    if (FALSE==bReadInRegister(TD_STATUS_ADR,TD_STATUS_SIZE+(TOUCHx_SIZE*2)))
    {
        return FALSE;
    }
    return TRUE;
}

/// Get the touch events from the controller
/// @param[in]    pInput = pointer to an array[2] of TCHINPUT, pCnt = pointer to count of touches
/// @param[out]   pInput = pointer to an array[2] of TCHINPUT, pCnt = pointer to count of touches
/// @retval       TRUE  success
/// @retval       FALSE communication error wit the touch controller,
/// @caveats      none
void vTouchPanelGetPoint(PTCHINPUT pInput,INT *pCnt)
{
    // static variables for previous evnts
    static BOOL bPrevTipSwitchID1   = FALSE;
    static BOOL bPrevTipSwitchID2   = FALSE;
    static INT  s_xPrevious;
    static INT  s_yPrevious;
    I2C_DEVICE sI2CDat;                                                 // data structure of the controller data
    BOOL bINTReady;
    INT     iFiCnt;
    BOOL bPrevTipSwitch_tmp;
    PTCHINPUT pInput_tmp;                                           // only for debug

    pInput_tmp = pInput;


    DEBUGMSG(ZONE_FUNCTION||ZONE_INFO, (TEXT("MultiTchAppDriv:vTouchPanelGetPoint() +\r\n")));

    // reset pointer for TOUCHEVENTF_PRIMARY flag handling
    *pCnt = 0;

    // clear all data for the call back to the MDD
    memset(pInput, 0, 2*sizeof(TCHINPUT));                          // clear array for call back function

    // Check if pen data are available If so, get the data.
    // Note that we always return data from the previous sample to avoid returning data nearest
    // the point where the pen is going up.  Data during light touches is not accurate and should
    // normally be rejected.  Without the ability to do pressure measurement, we are limited to
    // rejecting points near the beginning and end of the pen down period.
    bINTReady  = bTstINTFT5x06();
    if (bINTReady)
    {
        // get the data from the touch Controller
        vGetControllerData(&sI2CDat);

        // get the count of fingers
        iFiCnt = sI2CDat.u8NuFinger;

        // count of finges range vaid ?
        if ((0<iFiCnt)&&(3>iFiCnt))
        {
            //-------- generate the first data for the call back --------------------------------------------
            // choose the right previous state
            if (TOUCH_ID_1 == sI2CDat.u8TouchID1)
                bPrevTipSwitch_tmp = bPrevTipSwitchID1;
            else
                bPrevTipSwitch_tmp = bPrevTipSwitchID2;

            // Touch controller reports finger on screen
            if (sI2CDat.bTipSwitch1)
            {
                pInput->x       = sI2CDat.u16Px1;
                pInput->y       = sI2CDat.u16Py1;
                s_xPrevious     = sI2CDat.u16Px1;
                s_yPrevious     = sI2CDat.u16Py1;

                pInput->dwID    = sI2CDat.u8TouchID1;

                // previous already a finger on the screen
                if (bPrevTipSwitch_tmp)
                {
                    pInput->dwFlags = TOUCHEVENTF_MOVE|TOUCHEVENTF_INRANGE;
                }
                else
                {
                    // previous finger up, finger down detected
                    pInput->dwFlags = TOUCHEVENTF_DOWN|TOUCHEVENTF_INRANGE;
                    bPrevTipSwitch_tmp  = TRUE;
                }

                *pCnt = 1;
                // prepared for the next entry
                pInput++;
            }
            else
            {
                // previous already a finger on the screen, finger up detected
                if (bPrevTipSwitch_tmp)
                {
                    pInput->x       = s_xPrevious;
                    pInput->y       = s_yPrevious;

                    pInput->dwID    = sI2CDat.u8TouchID1;

                    pInput->dwFlags = TOUCHEVENTF_UP;
                    bPrevTipSwitch_tmp  = FALSE;

                    *pCnt = 1;
                    // prepared for the next entry
                    pInput++;
                }
            }

            // save the previous state
            if (TOUCH_ID_1 == sI2CDat.u8TouchID1)
                bPrevTipSwitchID1 = bPrevTipSwitch_tmp;
            else
                bPrevTipSwitchID2 = bPrevTipSwitch_tmp;


            //-------- generate the second data for the call back -------------------------------------
            // choose the right previous state
            if (TOUCH_ID_1 == sI2CDat.u8TouchID2)
                bPrevTipSwitch_tmp = bPrevTipSwitchID1;
            else
                bPrevTipSwitch_tmp = bPrevTipSwitchID2;

            // count of fingers range valid ?
            if (2==iFiCnt)
            {
                // Touch Controller reports finger on screen
                if (sI2CDat.bTipSwitch2)
                {
                    pInput->x       = sI2CDat.u16Px2;
                    pInput->y       = sI2CDat.u16Py2;
                    s_xPrevious     = sI2CDat.u16Px2;
                    s_yPrevious     = sI2CDat.u16Py2;

                    pInput->dwID    = sI2CDat.u8TouchID2;

                    // previous already a finger on the screen
                    if (bPrevTipSwitch_tmp)
                    {
                        pInput->dwFlags = TOUCHEVENTF_MOVE|TOUCHEVENTF_INRANGE;
                    }
                    else
                    {
                        // previous finger up, Finger down detected Now
                        pInput->dwFlags = TOUCHEVENTF_DOWN|TOUCHEVENTF_INRANGE;
                        bPrevTipSwitch_tmp  = TRUE;
                    }

                    *pCnt = 2;
                }
                else
                {
                    // previous already a finger on the screen, touch up detected
                    if (bPrevTipSwitch_tmp)
                    {
                        pInput->x       = s_xPrevious;
                        pInput->y       = s_yPrevious;

                        pInput->dwID    = sI2CDat.u8TouchID2;
                        pInput->dwFlags = TOUCHEVENTF_UP;
                        bPrevTipSwitch_tmp  = FALSE;

                        *pCnt = 2;
                    }
                    // else no finger on screen and no touch
                }

            }

            // save the previous state
            if (TOUCH_ID_1 == sI2CDat.u8TouchID2)
                bPrevTipSwitchID1 = bPrevTipSwitch_tmp;
            else
                bPrevTipSwitchID2 = bPrevTipSwitch_tmp;
        }

#if 0
        //-------- debug print ------------------------------------------------------------
        switch (sI2CDat.u8NuFinger)
        {
            case 0:
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() FT5x06 no touches\r\n")));
                break;
            case 1:
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() FT5x06 FirstTouch TipSwich(%d),TouchID(%d), X(%d), Y(%d)\r\n" ),sI2CDat.bTipSwitch1,sI2CDat.u8TouchID1,sI2CDat.u16Px1,sI2CDat.u16Py1));
                break;
            case 2:
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() FT5x06 FirstTouch TipSwich(%d),TouchID(%d), X(%d), Y(%d)\r\n" ),sI2CDat.bTipSwitch1,sI2CDat.u8TouchID1,sI2CDat.u16Px1,sI2CDat.u16Py1));
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() FT5x06 SecondTouch TipSwich(%d),TouchID(%d), X(%d), Y(%d)\r\n" ),sI2CDat.bTipSwitch2,sI2CDat.u8TouchID2,sI2CDat.u16Px2,sI2CDat.u16Py2));
                break;
            default:
                DEBUGMSG(ZONE_INFO,( TEXT("MultiTchAppDriv:vTouchPanelGetPoint() FT5x06 illegal  of fingers (%d)"),sI2CDat.u8NuFinger));
                break;
        }

        switch (*pCnt)
        {
            case 1:
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() Input[0] dwID(%d), dwFlags(%x), X(%d), Y(%d)\r\n" ),pInput_tmp->dwID, pInput_tmp->dwFlags, pInput_tmp->x, pInput_tmp->y));
                break;

            case 2:
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() Input[0] dwID(%d), dwFlags(%x), X(%d), Y(%d)\r\n" ),pInput_tmp->dwID, pInput_tmp->dwFlags, pInput_tmp->x, pInput_tmp->y));
                pInput_tmp++;
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() Input[1] dwID(%d), dwFlags(%x), X(%d), Y(%d)\r\n" ),pInput_tmp->dwID, pInput_tmp->dwFlags, pInput_tmp->x, pInput_tmp->y));
                break;

            default:
                DEBUGMSG(ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() illegal pCint (%d)\r\n" ),*pCnt));
        }
#endif
    }
    DEBUGMSG(ZONE_FUNCTION||ZONE_INFO,(TEXT("MultiTchAppDriv:vTouchPanelGetPoint() -\r\n")));
    return;
}

/// Reset off or Power on
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @caveats      none
void vResetOffFT5x06(void)
{
    if (0==dwRESET_Lin_Inv)                                             // Reset not inverted
    {
        SetGPIOLevel (dwRESET_GPIONumber,0);                            // Reset off or power on
    }
    else
    {
        SetGPIOLevel (dwRESET_GPIONumber,1);                            // Reset off / Power on
    }
    Sleep(dwRESET_PostDly);                                             // wait until the reset sequence is over
 }

/// Test the communication by reading out the vendor's chip id
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE communication error wit the touch controller,
/// @caveats      none
BOOL bCommunicationTestFT5x06(void)
{
    // attention: it is possible that a fast power on and off sequence can reset the touch controller and therefore the interrupt mode bit changed to recognize that, the interrupt mode is checked
    if (FALSE==bReadInRegister(ID_G_MODE_ADR,ID_G_MODE_SIZE))
    {
        return FALSE;
    }

    // get the interrupt status/mode
    if (ID_G_MODE_Polling == ucID_G_MODE_GET())
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}

