/// @file       TchContr_FT5x06.h
/// @copyright  Copyright (c) 2012 Toradex AG \n
///             All information contained herein is, and remains the property of Toradex Ag and its suppliers, if any. The intellectual\n
///             and technical concepts contained herein are proprietary to Toradex AG and its suppliers and may be covered by Swiss and\n
///             foreign patents, patents in process, and are protected by trade secret or copyright law. Dissemination of this information\n
///             or reproduction of this material is strictly forbidden unless prior written permission is obtained from Toradex AG.\n
/// @author     $Author$
/// @version    $Rev$
/// @date       $Date$
/// @brief      Function library for the touch controller FT5x06
///             GeneralError concept:\n
///             Every function, witch can detect errors, has as a return value BOOL ->  TRUE = ok, FALSE = error
///             The Details of the error itself is logged to the debug port.
/// @target     Colibri PXA270, PX3xx, T20 WinCE6, WinEC7
/// @caveats

#ifndef __TCHCONTRFT5X06_H__
#define __TCHCONTRFT5X06_H__

#include "unfd_multitchdrv.h"


#if (_WIN32_WCE == 0x600)
/*
 * Touch Input defines and functions
 */

/*
 * Touch input handle
 */
DECLARE_HANDLE(HTOUCHINPUT);

typedef struct tagTOUCHINPUT {
    LONG x;
    LONG y;
    HANDLE hSource;
    DWORD dwID;
    DWORD dwFlags;
    DWORD dwMask;
    DWORD dwTime;
    ULONG_PTR dwExtraInfo;
    DWORD cxContact;
    DWORD cyContact;
} TOUCHINPUT, *PTOUCHINPUT;
typedef TOUCHINPUT const * PCTOUCHINPUT;


/*
 * Conversion of touch input coordinates to pixels
 */
#define TOUCH_COORD_TO_PIXEL(l)         ((l) / 100)


/*
 * Touch input flag values (TOUCHINPUT.dwFlags)
 */

// Specifies that movement occurred. Cannot be combined with TOUCHEVENTF_DOWN.
#define TOUCHEVENTF_MOVE            0x0001

// Specifies that the corresponding touch point was established through a
// new contact. Cannot be combined with TOUCHEVENTF_MOVE or TOUCHEVENTF_UP.
#define TOUCHEVENTF_DOWN            0x0002

// Specifies that a touch point was removed.
#define TOUCHEVENTF_UP              0x0004

// Specifies that a touch point is in range. Must be set with all
// TOUCHEVENTF_DOWN and TOUCHEVENTF_MOVE events.
#define TOUCHEVENTF_INRANGE         0x0008

// Specifies that a touch point is designated as primary. Reserved for use
// by the touch proxy.
#define TOUCHEVENTF_PRIMARY         0x0010

// Specifies that this input should not be coalesced in the input stack.
#define TOUCHEVENTF_NOCOALESCE      0x0020

// Specifies that the touch point is associated with a pen contact.
#define TOUCHEVENTF_PEN             0x0040

// Specifies that the touch event came from the user's palm
#define TOUCHEVENTF_PALM            0x0080

// Specifies to the input system that it is not necessary to further
// calibrate the x and y coordinates of the sample.
#define TOUCHEVENTF_CALIBRATED      0x1000

// Specifies that the touch point is associated with a symmetric Touch
// H/W controller like the capacitive technology where the driver cannot
// accurately pair the X's and the Y's recorded to provide the actual X,Y
// touch contact points.
#define TOUCHEVENTF_SYMMETRIC       0x2000

/*
 * Touch input mask values (TOUCHINPUT.dwMask)
 */
#define TOUCHINPUTMASKF_TIMEFROMSYSTEM  0x0001  // the dwTime field contains a system generated value
#define TOUCHINPUTMASKF_EXTRAINFO       0x0002  // the dwExtraInfo field is valid
#define TOUCHINPUTMASKF_CONTACTAREA     0x0004  // the cxContact and cyContact fields are valid

#endif

#define dwSAMPLE_INT    15                                              // timeout wait for next sample interrupt
#define dwCOMTST_INT    5000                                            // timeout wait for communication test


//----- public methods declaration ------------------------------------------------------------------
/// Set the configuration data according to the registry entry or to the default value
/// These data are "INT port number", "RESET port number", "I2C slave address", "I2C speed",
/// The values are set to default or set by the entry in the registry (if a valid entry exists)
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @caveats      none
void vI2C_ReadConfiguration(void);

/// Init the I2C communication and the GIPO for the "INT" and "RST" pin of the touch controller
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE error, a message to the console shows the reason
/// @caveats      vI2C_ReadConfiguration() has to be executed
BOOL bI2C_Init(void);

/// Stop the I2C communication (release interrupt, mutex, stop the I2C driver etc)
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @retval       none
/// @caveats      none
void vI2C_DeInit(void);

/// Test the level of the INT line
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  INT line active
/// @retval       FALSE INT line passive
/// @caveats      none
BOOL bTstINTFT5x06(void);

/// Release the current interrupt and wait for the next interrupt with timeout
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  Interrupt
/// @retval       FALSE Timeout
/// @caveats      none
BOOL bWaitTouchDataINT(DWORD dwTimeout);

/// Start the touch controller (release the RST/PWR line) start the communication and and setup the\n
/// necessary registers in the touch controller
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE error, a message to the console shows the reason
/// @caveats      none
BOOL bInitFT5x06();

/// Reset on or power off the touch controller\n
/// Because the touch controller is mounted very closed to the display an ESD pulse could disturb the controller in a way that his is not working any more (not broken for ever)
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @caveats      none
void vDeInitFT5x06(void);

/// Read out the count of touches (TD_STATUS) and the data for the 2 touch from the  touch controller\n
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE communication error with the touch controller, a message to the console shows the reason
/// @caveats      none
BOOL bUpdateTouchCategoryFT5x06(void);

/// Get the touch events from the controller
/// @param[in]    pInput = pointer to an array[2] of TCHINPUT, pCnt = pointer to count of touches
/// @param[out]   pInput = pointer to an array[2] of TCHINPUT, pCnt = pointer to count of touches
/// @retval       TRUE  success
/// @retval       FALSE communication error wit the touch controller,
/// @caveats      none
void vTouchPanelGetPoint(PTCHINPUT pInput,INT *pCnt);

/// Reset off or Power on
/// @param[in]    none
/// @param[out]   none
/// @retval       none
/// @caveats      none
void vResetOffFT5x06(void);

/// Test the communication by reading out the vendor's chip id
/// @param[in]    none
/// @param[out]   none
/// @retval       TRUE  success
/// @retval       FALSE communication error wit the touch controller,
/// @caveats      none
BOOL bCommunicationTestFT5x06(void);

#endif //  __TCHCONTRFT5X06_H__

